package sun.java2d.pipe;

import java.awt.AlphaComposite;
import java.awt.Composite;
import sun.java2d.SurfaceData;
import sun.java2d.loops.CompositeType;
import sun.java2d.loops.SurfaceType;
import sun.java2d.loops.Blit;
import sun.java2d.loops.MaskBlit;

public abstract class BufferedMaskBlit extends MaskBlit
{
    private static final int ST_INT_ARGB = 0;
    private static final int ST_INT_ARGB_PRE = 1;
    private static final int ST_INT_RGB = 2;
    private static final int ST_INT_BGR = 3;
    private final RenderQueue rq;
    private final int srcTypeVal;
    private Blit blitop;
    
    protected BufferedMaskBlit(RenderQueue rq, SurfaceType srctype, CompositeType comptype, SurfaceType dsttype) {
        super(srctype, comptype, dsttype);
        this.rq = rq;
        if (srctype == SurfaceType.IntArgb) {
            this.srcTypeVal = 0;
        }
        else if (srctype == SurfaceType.IntArgbPre) {
            this.srcTypeVal = 1;
        }
        else if (srctype == SurfaceType.IntRgb) {
            this.srcTypeVal = 2;
        }
        else if (srctype == SurfaceType.IntBgr) {
            this.srcTypeVal = 3;
        }
        else {
            throw new InternalError("unrecognized source surface type");
        }
    }
    
    @Override
    public void MaskBlit(SurfaceData surfaceData, SurfaceData surfaceData2, Composite srcOver, Region region, int n, int n2, int n3, int n4, int n5, int n6, byte[] array, int n7, int n8) {
        if (n5 <= 0 || n6 <= 0) {
            return;
        }
        if (array == null) {
            if (this.blitop == null) {
                this.blitop = Blit.getFromCache(surfaceData.getSurfaceType(), CompositeType.AnyAlpha, this.getDestType());
            }
            this.blitop.Blit(surfaceData, surfaceData2, srcOver, region, n, n2, n3, n4, n5, n6);
            return;
        }
        else {
            if (((AlphaComposite)srcOver).getRule() != 3) {
                srcOver = AlphaComposite.SrcOver;
            }
            this.rq.lock();
            try {
                this.validateContext(surfaceData2, srcOver, region);
                RenderBuffer buffer = this.rq.getBuffer();
                this.rq.ensureCapacity(20 + n5 * n6 * 4);
                buffer.position((long)this.enqueueTile(buffer.getAddress(), buffer.position(), surfaceData, surfaceData.getNativeOps(), this.srcTypeVal, array, array.length, n7, n8, n, n2, n3, n4, n5, n6));
            }
            finally {
                this.rq.unlock();
            }
            return;
        }
    }
    
    private native int enqueueTile(long p0, int p1, SurfaceData p2, long p3, int p4, byte[] p5, int p6, int p7, int p8, int p9, int p10, int p11, int p12, int p13, int p14);
    
    protected abstract void validateContext(SurfaceData p0, Composite p1, Region p2);
}
